//
//  UtilTimerTest.cpp
//
//  Created  by Setsu on 5/27/2010.
//  Modified by Setsu on May 30, 2013.
//  Copyright 2010-2013 RoundSquare Inc. All rights reserved.
//
// Permission is hereby granted, free of charge, to any person or organization
// obtaining a copy of the software and accompanying documentation covered by
// this license (the "Software") to use, reproduce, display, distribute,
// execute, and transmit the Software, and to prepare derivative works of the
// Software, and to permit third-parties to whom the Software is furnished to
// do so, all subject to the following:
// 
// The copyright notices in the Software and this entire statement, including
// the above license grant, this restriction and the following disclaimer,
// must be included in all copies of the Software, in whole or in part, and
// all derivative works of the Software, unless such copies or derivative
// works are solely in the form of machine-executable object code generated by
// a source language processor.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
// SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
// FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
//

//----------------------------------------
//	include
//----------------------------------------
#include <Poco/Util/Timer.h>
#include <Poco/Util/TimerTask.h>
#include <Poco/Util/TimerTaskAdapter.h>
#include <Poco/Event.h>
#include <Poco/Timestamp.h>
#include <Poco/Format.h>

#include <string>

#include "ScopedLogMessage.h"
#include "PrepareConsoleLogger.h"

//----------------------------------------
//	MyTimerTask
//----------------------------------------
class MyTimerTask
{
public:
	MyTimerTask(ScopedLogMessage& msg, Poco::Event& event) :
		m_msg(msg)
	,	m_event(event)
	{
	}
	void onTimer(Poco::Util::TimerTask& task)
	{
		m_msg.Message("  MyTimerTask::onTimer() called");
		Poco::Thread::sleep(100);	// 100msec
		m_event.set();
	}
private:
	ScopedLogMessage&	m_msg;
	Poco::Event&		m_event;
};

//----------------------------------------
//	TestSchedule
//----------------------------------------
void TestSchedule(ScopedLogMessage& msg, Poco::Util::Timer& timer)
{
	msg.Message("--- schedule ---");

	const Poco::Timestamp::TimeDiff kTimeDiff = 500000;	// 500msec

	Poco::Event event;
	MyTimerTask task(msg, event);
	Poco::Util::TimerTask::Ptr pTask = new Poco::Util::TimerTaskAdapter<MyTimerTask>(task, &MyTimerTask::onTimer);

	Poco::Timestamp time;
	time += kTimeDiff;
	timer.schedule(pTask, time);
	
	event.wait();
	msg.Message(Poco::format("    execution delay from scheduled: %Ldusec"
							, pTask->lastExecution().epochMicroseconds()-time.epochMicroseconds()));
}

//----------------------------------------
//	WaitAndShowTime
//----------------------------------------
Poco::Timestamp::TimeVal WaitAndShowTime(	ScopedLogMessage& msg
										,	Poco::Event& event
										,	Poco::Util::TimerTask::Ptr& pTask
										,	const std::string& text
										,	Poco::Timestamp::TimeVal refTime)
{
	event.wait();
	Poco::Timestamp::TimeVal val = pTask->lastExecution().epochMicroseconds();
	msg.Message(Poco::format("%s %7.3fmsec", text, 0.001*(val-refTime)));
	return val;
}

//----------------------------------------
//	TestFixedRate
//----------------------------------------
void TestFixedRate(ScopedLogMessage& msg, Poco::Util::Timer& timer)
{
	msg.Message("--- schedule fixed rate ---");

	Poco::Event event;
	MyTimerTask task(msg, event);
	Poco::Util::TimerTask::Ptr pTask = new Poco::Util::TimerTaskAdapter<MyTimerTask>(task, &MyTimerTask::onTimer);

	Poco::Timestamp time;
	timer.scheduleAtFixedRate(pTask, 500, 500);	// 500msec

	Poco::Timestamp::TimeVal val;
	val = WaitAndShowTime(msg, event, pTask, "    delay   :", time.epochMicroseconds());
	for(int i=0; i<2; ++i)
	{
		val = WaitAndShowTime(msg, event, pTask, "    interval:", val);
	}
	pTask->cancel();
}

//----------------------------------------
//	TestInterval
//----------------------------------------
void TestInterval(ScopedLogMessage& msg, Poco::Util::Timer& timer)
{
	msg.Message("--- schedule interval ---");

	Poco::Event event;
	MyTimerTask task(msg, event);
	Poco::Util::TimerTask::Ptr pTask = new Poco::Util::TimerTaskAdapter<MyTimerTask>(task, &MyTimerTask::onTimer);

	Poco::Timestamp time;
	timer.schedule(pTask, 500, 500);	// 500msec

	Poco::Timestamp::TimeVal val;
	val = WaitAndShowTime(msg, event, pTask, "    delay   :", time.epochMicroseconds());
	for(int i=0; i<2; ++i)
	{
		val = WaitAndShowTime(msg, event, pTask, "    interval:", val);
	}
	pTask->cancel();
}

//----------------------------------------
//	main
//----------------------------------------
int main(int /*argc*/, char** /*argv*/)
{
	PrepareConsoleLogger logger(Poco::Logger::ROOT, Poco::Message::PRIO_INFORMATION);

	ScopedLogMessage msg("UtilTimerTest ", "start", "end");

	Poco::Util::Timer timer;

	TestSchedule(msg, timer);
	TestFixedRate(msg, timer);
	TestInterval(msg, timer);

	return 0;
}
